@@ -1225,6 +1225,13 @@ prolog_message(pack(duplicate(Entry, OldDir, Dir))) -->
 prolog_message(pack(no_arch(Entry, Arch))) -->
 	[ 'Package ~w: no binary for architecture ~w'-[Entry, Arch] ].
 
+		 /*******************************
+		 *	       MISC		*
+		 *******************************/
+
+prolog_message(null_byte_in_path(Component)) -->
+	[ '0-byte in PATH component: ~p (skipped directory)'-[Component] ].
+
 
 		 /*******************************
 		 *	PRINTING MESSAGES	*
@@ -1285,8 +1292,10 @@ print_system_message(Term, Kind, Lines) :-
 		  ],
 		  AllLines),
 	msg_property(Kind, stream(Stream)),
+	ignore(stream_property(Stream, position(Pos))),
 	print_message_lines(Stream, AllLines),
-	(   msg_property(Kind, wait(Wait)),
+	(   \+ stream_property(Stream, position(Pos)),
+	    msg_property(Kind, wait(Wait)),
 	    Wait > 0
 	->  sleep(Wait)
 	;   true
@@ -1455,6 +1464,7 @@ append_args(Args0, Args1, Args) :-
 %	True for messages that must be printed only once.
 
 print_once(compatibility(_), _).
+print_once(null_byte_in_path(_), _).
 
 %%	must_print(+Level, +Message)
 %
