@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2009-2014, University of Amsterdam
+    Copyright (C): 2009-2015, University of Amsterdam
 			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
@@ -84,6 +84,7 @@
 :- use_module(library(http/http_wrapper)).
 :- use_module(library(http/http_path)).
 :- use_module(library(http/html_head)).
+:- use_module(library(http/jquery)).
 :- use_module(library(debug)).
 :- use_module(library(apply)).
 :- use_module(library(pairs)).
@@ -193,13 +194,6 @@ extracting module doc_wiki.pl into HTML+CSS.
 		   requires([ pldoc_resource('pldoc.css')
 			    ])
 		 ]).
-:- if(\+html_current_resource(jquery)).
-:- html_resource(jquery,
-		 [ virtual(true),
-		   requires([ pldoc_resource('jquery.js')
-			    ])
-		 ]).
-:- endif.
 :- html_resource(pldoc_resource('pldoc.js'),
 		 [ requires([ jquery
 			    ])
@@ -1361,12 +1355,21 @@ anchored_pred_head(Head, Done0, Done, Options) -->
 	->  { Done = Done0 },
 	    pred_head(Head)
 	;   html([ span(style('float:right'),
-			\pred_edit_button(Head, Options)),
+			\pred_edit_or_source_button(Head, Options)),
 		   a(name=Name, \pred_head(Head))
 		 ]),
 	    { Done = [PI|Done0] }
 	).
 
+
+pred_edit_or_source_button(Head, Options) -->
+	{ option(edit(true), Options) }, !,
+	pred_edit_button(Head, Options).
+pred_edit_or_source_button(Head, Options) -->
+	{ option(source_link(true), Options) }, !,
+	pred_source_button(Head, Options).
+pred_edit_or_source_button(_, _) --> [].
+
 %%	pred_edit_button(+PredIndicator, +Options)// is det.
 %
 %	Create a button for editing the given predicate.  Options
@@ -1980,8 +1983,8 @@ in_file(Module, Head, File) :-
 	;   Primary = Module
 	),
 	module_property(Primary, file(File)).
-in_file(_, Head, File) :-
-	source_file(Head, File).
+in_file(Module, Head, File) :-
+	predicate_property(Module:Head, file(File)).
 in_file(Module, Head, File) :-
 	current_module(Module),
 	source_file(Module:Head, File).
