@@ -24,6 +24,7 @@
 #include "pl-incl.h"
 #include <ctype.h>
 #include "pl-ctype.h"
+#include <errno.h>
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This module defines:
@@ -398,24 +399,18 @@ do_char_type(term_t chr, term_t class, control_t h, int how)
     goto error;
 
   for(;;)
-  { int rval;
+  { int rval = (*gen->class->test)((wint_t)gen->current);
 
-    if ( (rval = (*gen->class->test)((wint_t)gen->current)) )
-    { if ( gen->do_enum & ENUM_CHAR )
+    if ( (gen->class->arity == 0 && rval) ||
+	 (gen->class->arity  > 0 && rval >= 0) )
+    { if ( (gen->do_enum & ENUM_CHAR) )
       { if ( !PL_unify_char(chr, gen->current, how) )
 	{ if ( LD->exception.term )
 	    goto error;
 	  goto next;
 	}
       }
-      if ( gen->class->arity > 0 )
-      { if ( rval < 0 ||
-	     !unify_char_type(class, gen->class, rval, how) )
-	{ if ( LD->exception.term )
-	    goto error;
-	  goto next;
-	}
-      } else if ( gen->do_enum & ENUM_CLASS )
+      if ( (gen->do_enum & ENUM_CLASS) || gen->class->arity > 0 )
       { if ( !unify_char_type(class, gen->class, rval, how) )
 	{ if ( LD->exception.term )
 	    goto error;
@@ -870,7 +865,7 @@ IOENC
 initEncoding(void)
 { GET_LD
 
-  if ( LD )
+  if ( HAS_LD )
   { if ( !LD->encoding )
     { char *enc;
 
