@@ -1575,7 +1575,7 @@ DirName(const char *f, char *dir)
 { const char *base, *p;
 
   for(base = p = f; *p; p++)
-  { if (*p == '/' && p[1] != EOS )
+  { if ( *p == '/' )
       base = p;
   }
   if ( base == f )
@@ -1932,7 +1932,7 @@ PopTty(IOSTREAM *s, ttybuf *buf, int do_free)
   { GET_LD
     int fd;
 
-    if ( (!LD || truePrologFlag(PLFLAG_TTY_CONTROL)) &&
+    if ( (!HAS_LD || truePrologFlag(PLFLAG_TTY_CONTROL)) &&
 	 (fd = Sfileno(s)) >= 0 )
       rc = SetTtyState(fd, &TTY_STATE(buf));
 
