@@ -3657,6 +3657,12 @@ PL_foreign_control(control_t h)
 { return ForeignControl(h);
 }
 
+predicate_t				/* = Procedure */
+PL_foreign_context_predicate(control_t h)
+{ Definition def = h->predicate;
+
+  return isCurrentProcedure(def->functor->functor, def->module);
+}
 
 static int
 copy_exception(term_t ex, term_t bin ARG_LD)
@@ -3755,8 +3761,10 @@ void
 PL_clear_exception(void)
 { GET_LD
 
-  resumeAfterException(TRUE, LD->outofstack);
-  LD->outofstack = NULL;
+  if ( exception_term )
+  { resumeAfterException(TRUE, LD->outofstack);
+    LD->outofstack = NULL;
+  }
 }
 
 
@@ -3857,8 +3865,8 @@ resolveModule(const char *module)
     return PL_new_module(PL_new_atom(module));
   else
   { GET_LD
-    return (LD && environment_frame ? contextModule(environment_frame)
-	                            : MODULE_user);
+    return (HAS_LD && environment_frame ? contextModule(environment_frame)
+                                        : MODULE_user);
   }
 }
 
@@ -4033,7 +4041,7 @@ PL_raise(int sig)
 
 int
 PL_pending__LD(int sig ARG_LD)
-{ if ( sig > 0 && sig <= MAXSIGNAL && LD )
+{ if ( sig > 0 && sig <= MAXSIGNAL && HAS_LD )
   { int off  = (sig-1)/32;
     int mask = 1 << ((sig-1)%32);
 
@@ -4046,7 +4054,7 @@ PL_pending__LD(int sig ARG_LD)
 
 int
 PL_clearsig__LD(int sig ARG_LD)
-{ if ( sig > 0 && sig <= MAXSIGNAL && LD )
+{ if ( sig > 0 && sig <= MAXSIGNAL && HAS_LD )
   { int off  = (sig-1)/32;
     int mask = 1 << ((sig-1)%32);
 
@@ -4542,7 +4550,7 @@ PL_query(int query)
     case PL_QUERY_ENCODING:
     { GET_LD
 
-      if ( LD )
+      if ( HAS_LD )
 	return LD->encoding;
       return PL_local_data.encoding;	/* Default: of main thread? */
     }
