@@ -35,6 +35,8 @@ COMMON(void)		assignAttVar(Word av, Word value ARG_LD);
 COMMON(int)		saveWakeup(wakeup_state *state, int forceframe ARG_LD);
 COMMON(void)		restoreWakeup(wakeup_state *state ARG_LD);
 COMMON(int)		PL_get_attr__LD(term_t t, term_t a ARG_LD);
+COMMON(int)		on_attvar_chain(Word avp);
+COMMON(Word)		alloc_attvar(ARG1_LD);
 
 /* pl-gvar.c */
 
@@ -58,6 +60,7 @@ COMMON(void)		updateAlerted(PL_local_data_t *ld);
 COMMON(int)		raiseSignal(PL_local_data_t *ld, int sig);
 COMMON(Module)		contextModule(LocalFrame fr);
 COMMON(void)		setContextModule(LocalFrame fr, Module context);
+COMMON(int)		existingChoice(Choice ch ARG_LD);
 
 /* pl-stream.c */
 
@@ -383,7 +386,7 @@ COMMON(int)		compareAtoms(atom_t a1, atom_t a2);
 COMMON(intptr_t)	skip_list(Word l, Word *tailp ARG_LD);
 COMMON(intptr_t)	lengthList(term_t list, int errors);
 COMMON(int)		is_acyclic(Word p ARG_LD);
-COMMON(int)		numberVars(term_t t, nv_options *opts, int n ARG_LD);
+COMMON(intptr_t)	numberVars(term_t t, nv_options *opts, intptr_t n ARG_LD);
 COMMON(int)		duplicate_term(term_t in, term_t copy ARG_LD);
 COMMON(word)		stringToList(char *s);
 COMMON(foreign_t)	pl_sub_atom(term_t atom,
@@ -419,7 +422,6 @@ COMMON(void)		cleanupPrologFlags(void);
 
 /* pl-pro.c */
 COMMON(word)		pl_break(void);
-COMMON(word)		pl_notrace1(term_t goal);
 COMMON(int)		currentBreakLevel(void);
 COMMON(int)		callProlog(Module module, term_t goal, int flags, term_t *ex);
 COMMON(int)		abortProlog(void);
