@@ -468,8 +468,7 @@ them.  Descriptions:
 #define PLMAXTAGGEDINT		(-PLMINTAGGEDINT - 1)
 #define PLMINTAGGEDINT32	(-(intptr_t)((word)1<<(32-LMASK_BITS-1)))
 #define PLMAXTAGGEDINT32	(-PLMINTAGGEDINT32 - 1)
-#define inTaggedNumRange(n)	(((n)&~PLMAXTAGGEDINT) == 0 || \
-				 ((n)&~PLMAXTAGGEDINT) == ~PLMAXTAGGEDINT)
+#define inTaggedNumRange(n)	(valInt(consInt(n)) == (n))
 #define PLMININT		(((int64_t)-1<<(INT64BITSIZE-1)))
 #define PLMAXINT		(-(PLMININT+1))
 #if SIZEOF_WCHAR_T == 2
@@ -1335,6 +1334,9 @@ typedef enum
   DBG_ALL				/* switch on globally */
 } debug_type;
 
+#define SKIP_VERY_DEEP	  ((size_t)-1)	/* deep skiplevel */
+#define SKIP_REDO_IN_SKIP (SKIP_VERY_DEEP-1)
+
 struct clause_choice
 { ClauseRef	cref;			/* Next clause reference */
   word		key;			/* Search key */
@@ -1862,8 +1864,11 @@ typedef enum
   AV_ERROR
 } av_action;
 
+#define NV_ERROR (PLMINTAGGEDINT-1)
+
 typedef struct
 { functor_t functor;			/* Functor to use ($VAR/1) */
+  intptr_t  offset;			/* offset */
   av_action on_attvar;			/* How to handle attvars */
   int	    singletons;			/* Write singletons as $VAR('_') */
   int	    numbered_check;		/* Check for already numbered */
@@ -1990,7 +1995,6 @@ typedef struct
 #define PROCEDURE_setup_call_catcher_cleanup4 \
 				(GD->procedures.setup_call_catcher_cleanup4)
 #define PROCEDURE_dwakeup1		(GD->procedures.dwakeup1)
-#define PROCEDURE_call_residue_vars2	(GD->procedures.call_residue_vars2)
 #define PROCEDURE_dthread_init0		(GD->procedures.dthread_init0)
 #define PROCEDURE_exception_hook4	(GD->procedures.exception_hook4)
 #define PROCEDURE_dc_call_prolog	(GD->procedures.dc_call_prolog0)
