@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2012, University of Amsterdam
+    Copyright (C): 1985-2015, University of Amsterdam
 			      VU University Amsterdam
 
     This library is free software; you can redistribute it and/or
@@ -236,7 +236,7 @@ void
 updateAlerted(PL_local_data_t *ld)
 { int mask = 0;
 
-  if ( is_signalled(ld) )			mask |= ALERT_SIGNAL;
+  if ( is_signalled(PASS_LDARG1(ld)) )		mask |= ALERT_SIGNAL;
 #ifdef O_PROFILE
   if ( ld->profile.active )			mask |= ALERT_PROFILE;
 #endif
@@ -1208,6 +1208,21 @@ TrailAssignment__LD(Word p ARG_LD)
 }
 
 
+#ifdef O_ATTVAR
+static void
+reclaim_attvars(Word after ARG_LD)
+{ while ( LD->attvar.attvars >= after )
+  { word w = *LD->attvar.attvars;
+
+    if ( isVar(w) )
+      LD->attvar.attvars = NULL;
+    else
+      LD->attvar.attvars = unRef(w);
+  }
+}
+#endif
+
+
 static inline void
 __do_undo(mark *m ARG_LD)
 { TrailEntry tt = tTop;
@@ -1220,6 +1235,9 @@ __do_undo(mark *m ARG_LD)
     { DEBUG(2, Sdprintf("Undoing a trailed assignment\n"));
       tt--;
       *tt->address = trailVal(p);
+      DEBUG(CHK_SECURE,
+	    if ( isAttVar(*tt->address) )
+	      assert(on_attvar_chain(tt->address)));
       assert(!(*tt->address & (MARK_MASK|FIRST_MASK)));
     } else
       setVar(*p);
@@ -1228,9 +1246,11 @@ __do_undo(mark *m ARG_LD)
   tTop = mt;
   if ( LD->frozen_bar > m->globaltop )
   { DEBUG(CHK_SECURE, assert(gTop >= LD->frozen_bar));
+    reclaim_attvars(LD->frozen_bar PASS_LD);
     gTop = LD->frozen_bar;
   } else
-  { gTop = m->globaltop;
+  { reclaim_attvars(m->globaltop PASS_LD);
+    gTop = m->globaltop;
   }
 }
 
@@ -1254,6 +1274,7 @@ do_undo(mark *m)
    our block is not the real memory pointer.
 */
 
+#ifdef O_PLMT
 static Definition
 localDefinition(Definition def ARG_LD)
 { unsigned int tid = LD->thread.info->pl_tid;
@@ -1282,7 +1303,6 @@ localDefinition(Definition def ARG_LD)
   return v->blocks[idx][tid];
 }
 
-
 void
 destroyLocalDefinition(Definition def, unsigned int tid)
 { size_t idx = MSB(tid);
@@ -1293,7 +1313,7 @@ destroyLocalDefinition(Definition def, unsigned int tid)
   v->blocks[idx][tid] = NULL;
   destroyDefinition(local);
 }
-
+#endif
 
 Definition
 getProcDefinition__LD(Definition def ARG_LD)
@@ -1988,7 +2008,7 @@ chp_chars(Choice ch)
 #endif
 
 
-static int
+int
 existingChoice(Choice ch ARG_LD)
 { if ( onStack(local, ch) && onStack(local, ch->frame) &&
        (int)ch->type >= 0 && (int)ch->type <= CHP_DEBUG )
