@@ -622,14 +622,14 @@ dispatchFile(const char *name)
 
     for( ; d->extension; d++ )
     { if ( strfeq(d->extension, ext) )
-      { if ( d->list == &plfiles )
+      { if ( d->list == &plfiles || d->list == &qlfiles )
 	  nostate = FALSE;
 	appendArgList(d->list, name);
 	return TRUE;
       }
     }
     if ( soext && strfeq(soext, ext) )
-    { if ( d->list == &plfiles )
+    { if ( d->list == &plfiles || d->list == &qlfiles )
 	nostate = FALSE;
       appendArgList(&libs, name);
       return TRUE;
@@ -1439,6 +1439,12 @@ createSavedState()
     quoted_name(plfiles.list[n], e);
     e += strlen(e);
   }
+  for(n=0; n<qlfiles.size; n++)
+  { if ( n > 0 )
+      *e++ = ',';
+    quoted_name(qlfiles.list[n], e);
+    e += strlen(e);
+  }
   strcpy(e, "]),qsave_program(");
   e += strlen(e);
   quoted_name(pltmp, e);
